// data
let file;
let data;

// fonts
let robotoBold, robotoLight, robotoMedium, robotoRegular, robotoThin;

// background image
let backgroundImage;

// visualization
let counterData = 0;
let zoom = 10.0;
let speed = 0;

let hintScroll = true;
let hintSpace = true;
let play = false;

let wind = [];
let sun = [];
let temp = [];

let modulator = [];
let modulator2 = [];



// sound design
let synths = [];
let synths2 = [];
let synths3 = [];
let volume = 0.5;
let counterSynths = 0;
let counterSynths2 = 0;
let counterSynths3 = 0;

let tune = 55;
let tune2 = 35;
let level = 10;







// preload
function preload() {

  // load json file with weather data
  file = loadJSON("https://raw.githubusercontent.com/HybridThingsLab/course-interaction-design/master/Block_I/data/DWD-Augsburg-2009-2019.json");

  // load fonts
  robotoBold = loadFont('fonts/Roboto-Bold.ttf');
  robotoLight = loadFont('fonts/Roboto-Light.ttf');
  robotoMedium = loadFont('fonts/Roboto-Medium.ttf');
  robotoRegular = loadFont('fonts/Roboto-Regular.ttf');
  robotoThin = loadFont('fonts/Roboto-Thin.ttf');

  // load image
  backgroundImage = loadImage('data/Background.jpg');


}

// setup
function setup() {

  // canvas
  createCanvas(1280, 800);
 
  // set volume
  Tone.Master.volume.value = volume;

  // sound design
  for (let i = 0; i < 10; i++) {
    let synth = new Tone.Synth({
      oscillator: {
        type: "sine",
        modulationFrequency: 0.2
      },
      envelope: {
        attack: 0.02,
        decay: 0.1,
        sustain: 0.005, // change here!!!
        release: 0.9,
      }
    });
    // add to array
    synths.push(synth);
    // connect synth to Tone Master
    synths[i].connect(Tone.Master);

    let synth2 = new Tone.Synth({
      oscillator: {
        type: "triangle8",
        modulationFrequency: 0.2
      },
      envelope: {
        attack: 0.05,
        decay: 0.1,
        sustain: 0.008, // change here!!!
        release: 0.9,
      }
    });
    // add to array
    synths2.push(synth2);
    // connect synth to Tone Master
    synths2[i].connect(Tone.Master);
    
    let synth3 = new Tone.Synth({
      oscillator: {
        type: 'square2',
        modulationFrequency: 0.2
      },
      envelope: {
        attack: 0.02,
        decay: 0.02,
        sustain: 0.005, // change here!!!
        release: 0.9,
      }
    });
    // add to array
    synths3.push(synth3);
    // connect synth to Tone Master
    synths3[i].connect(Tone.Master);
  }

  

  // get all weather data
  data = file.data;

  // create sound levels
  for(let i = 1; i < data.length; i++) {
    // sound level nr. 1
    let x = 590;
    let v = lerp(data[i-1].FM, data[i].FM, 0.5);
    let z = 10;
    let p = new SoundLevel(x, v, z);
    wind.push(p);

    // sound level nr. 2
    let x2 = 895;
    let v2 = lerp(data[i-1].SDK, data[i].SDK, 0.5);
    let z2 = 5;
    let p2 = new SoundLevel(x2, v2, z2);
    sun.push(p2);

    // sound level nr. 3
    let x4 = 1200;
    let v4 = lerp(data[i-1].TMK, data[i].TMK, 0.5);
    let z4 = 3;
    let p4 = new SoundLevel(x4, v4, z4);
    temp.push(p4);
  }

  // create modulators
  for(let i = 0; i < 3; i++) {
    let modulatorX = 490 + i * 300;
    let modulatorY = height/2+100;
    let modulatorB = 1;
    let modulatorH = 250;
    let mod = new Modulator(modulatorX, modulatorY, modulatorB, modulatorH);
    modulator.push(mod);

    let modulatorX2 = 490 + i * 300;
    let modulatorY2 = height/2+100;
    let modulatorB2 = 30;
    let modulatorH2 = 60;
    let mod2 = new Modulator(modulatorX2, modulatorY2, modulatorB2, modulatorH2);
    modulator2.push(mod2);
  }



}

// draw
function draw() {

  // clear background
  background(backgroundImage);

  smooth();
  strokeWeight(2);
  stroke(0,255,0);
  noFill();
  rectMode(CORNER);
  //rect(55, 325, 330, 420 );
  
  // show hints
 // if(hintScroll || hintSpace) {
    fill(255);
    noStroke();
    textSize(28);
    textAlign(LEFT);
    textFont(robotoBold);
    text('I N S T R U C T I O N S', 56, 350);
    textSize(18);
    textFont(robotoRegular);
    text('find out what modulators do', 56, 470);
 // }

  //if(hintSpace) {
    fill(255);
    noStroke();
    textSize(18);
    textAlign(LEFT);
    textFont(robotoRegular);
    text("press space to start and pause", 56, 395);
    text("use numbers to change year", 56, 420);
  //}

 // if(hintScroll) {
    fill(255);
    noStroke();
    textSize(18);
    textAlign(LEFT);
    text("scroll to change speed", 56, 445);
  //}

 

  if(!play) {
    speed = 0;
  }

  // show current data
  fill(255);
  noStroke();
  textSize(28);
  textFont(robotoBold);
  text('C U R R E N T   D A T A', 56, 550);
  textSize(20);
  textAlign(LEFT);
  textFont(robotoRegular);
  text('DATE  —  ' + data[counterData].MESS_DATUM.substr(8, 2) + ' / ' + data[counterData].MESS_DATUM.substr(5,2) + ' / ' + data[counterData].MESS_DATUM.substr(0,4), 56, 600);
  text('WINDSPEED  —  ' + round(data[counterData].FM) + '  m / s', 56, 650);
  text('SUNSHINE  —  ' + round(data[counterData].SDK) + '  h / d', 56, 700);
  text('TEMPERATURE  —  ' + round(data[counterData].TMK) + '  °c', 56, 750);

  

  wind[counterData].show();
  sun[counterData].show();
  //rain[counterData].show();
  temp[counterData].show();

  for(let i = 0; i < modulator.length; i++) {
    modulator[i].render();
    modulator2[i].render();
    
  } 

 
 /* textAlign(RIGHT);
  fill(255);
  text(data[counterData].FM + " m/s", 210 + 28, height / 2 + 30); 

  
  text(data[counterData].SDK + ' h/d', 430 + 28, height / 2 + 30);

  

  
  text(data[counterData].TMK + ' °C', 870 + 28, height / 2 + 30); */

  //sound design
  // play note
  if ((frameCount % speed) == 0 && (play)) {
    playNote(tune, tune2, level);
  }
  

  // counter data
  // just every x frames
  if ((frameCount % speed) == 0) {
    counterData++;
  }
  if (counterData > data.length-2) {
    counterData = 0;
  }

  

}

function mouseDragged() {
  for(let i = 0; i < modulator2.length; i++) {
    modulator2[i].drag(pmouseX, pmouseY);
    
    
  }
  if(mouseX > 490-15 && mouseX < 490+15 && mouseY > 375 && mouseY < 625) {
    tune = map(mouseY, 375, 625, 100, 10);
  }
  if(mouseX > 790-15 && mouseX < 790+15 && mouseY > 375 && mouseY < 625) {
    level = map(mouseY, 375, 625, 20, 0);
  }

  if(mouseX > 1090-15 && mouseX < 1090+15 && mouseY > 375 && mouseY < 625) {
    tune2 = map(mouseY, 375, 625, 80, -10)
  }
  
  
}

function mouseWheel(event) {
  print(event.delta);
  if(event.delta < 0) {
    speed--;
    if(speed < 4) {
      speed = 4;
    }
  }

  if(event.delta > 0) {
    speed++;
    if(speed > 60) {
      speed = 60;
    }
  }
  hintScroll = false;
  
}

function mouseMoved() {
  for(let i = 0; i < modulator2.length; i++) {
    modulator2[i].hover(pmouseX, pmouseY);
  }
}

function keyReleased() {
  // start and stop animation and sound with space key
  if(keyCode == '32') { 
    play = !play;
    speed = 4;
    playNote(tune, tune2, level);    
  }

  // navigate through years with number keys
  if(keyCode == '49') { // 1 2010
    counterData = 365;
  }
  if(keyCode == '50') { // 2 2011
    counterData = 365+365;
  }
  if(keyCode == '51') { // 3 2012
    counterData = 365+365+365;
  }
  if(keyCode == '52') { // 4 2013
    counterData = 365+365+365+366;
  }
  if(keyCode == '53') { // 5 2014
    counterData = 365+365+365+366+365;
  }
  if(keyCode == '54') { // 6 2015
    counterData = 365+365+365+366+365+365;
  }
  if(keyCode == '55') { // 7 2016
    counterData = 365+365+365+366+365+365+365;
  }
  if(keyCode == '56') { // 8 2017
    counterData = 365+365+365+366+365+365+365+366;
  }
  if(keyCode == '57') { // 9 2018
    counterData = 365+365+365+366+365+365+365+366+365;
  }
  
  
  hintSpace = false;
}


// sound design
function playNote(tune, tune2, level) {
  // or number
  let freq = map(round(data[counterData].FM), 0, 20, 20, 100);
  let freq2 = map(round(data[counterData].SDK), 0, 20, 30, 100);
  let freq3 = map(round(data[counterData].TMK), -20, 45, 10, 80);
  //let randomFactor = 10; //int(random(10, 50));
  synths[counterSynths].triggerAttackRelease(freq * tune, 2);
  if (data[counterData].SDK > level) { 
    synths2[counterSynths2].triggerAttackRelease(freq2 * 2, 3);
  }
  synths3[counterSynths3].triggerAttackRelease(freq3 * tune2, 2);
  
  // counter synths
  counterSynths++;
  counterSynths2++;
  counterSynths3++;
  if(counterSynths > 4) {
    counterSynths = 0;
  }
  if(counterSynths2 > 2) {
    counterSynths2 = 0;
  }
  if(counterSynths3 > 8) {
    counterSynths3 = 0;
  }
}

class SoundLevel {
  constructor(x, v, z) {
    this.x = x;
    this.value = v;
    this.zoom = z;

    this.level = abs(this.value) * this.zoom;
  }

  show() {
    noStroke();

    if(this.level < 1) {
      this.level = 10;
    }
    
    fill(60, 170, 140);
    for(let i = 0; i < this.level; i++) {
      
      
      let y = height-120-i*8;
      rectMode(CENTER);
      rect(this.x, y, 80, 8);

      if(y <= height/2-100) {
        fill(255, 220, 100);
      } 
      if(y <= height/2-180) {
        fill(255, 80, 0);
      }
      if(y <= height/2-230){
        fill(200, 60, 0);
      }

      if(y <= height/2-260) {
        fill(0);
      }

      if(this.value < 1) {
        this.value = 1;
        
      }

      
    }
    
    
    
  }


}

class Modulator {
  constructor(x, y, b, h) {
    this.x = x;
    this.y = y;
    this.b = b;
    this.h = h;
    this.value = 255;
    
  }

  render() {
    noStroke();
    fill(this.value);
    rectMode(CENTER);
    rect(this.x, this.y, this.b, this.h);
  }

  drag(xpos, ypos) {
    let d = dist(xpos, ypos, this.x, this.y);
    if(d < this.h/2) {
      this.y = ypos;
      if(this.y < height/2+100-125+this.h/2) {
        this.y = height/2+100-125+this.h/2;
      } else if(this.y > height/2+100+125-this.h/2) {
        this.y = height/2+100+125-this.h/2;
      }
    }
    //console.log(this.y);

  }

  hover(xpos, ypos) {
    let d = dist(xpos, ypos, this.x, this.y);
    if(d < this.h/2) {
      this.b = 35;
      this.h = 65;
    } else {
      this.b = 30;
      this.h = 60;
    }
  }

  changeFreq() {
    
    let rand = map(this.y, 545, 655, 10, 100); //int(random(10, 50));
    return rand;
    
  
    // counter synths
    
  }
}